#!/usr/bin/env python

# Written by John Hoffman
# see LICENSE.txt for license information

from BitTornado import PSYCO
if PSYCO.psyco:
    try:
        import psyco
        assert psyco.__version__ >= 0x010100f0
        psyco.full()
    except:
        pass

from BitTornado.launchmanycore import LaunchMany
from BitTornado.download_bt1 import defaults, get_usage
from BitTornado.parseargs import parseargs
from threading import Event
from sys import argv, exit
import sys, os
from BitTornado import version

try:
    import curses
    import curses.panel
    from curses.wrapper import wrapper as curses_wrapper
    from signal import signal, SIGWINCH 
except:
    print 'Textmode GUI initialization failed, cannot proceed.'
    print
    print 'This download interface requires the standard Python module ' \
       '"curses", which is unfortunately not available for the native ' \
       'Windows port of Python. It is however available for the Cygwin ' \
       'port of Python, running on all Win32 systems (www.cygwin.com).'
    print
    print 'You may still use "btdownloadheadless.py" to download.'
    sys.exit(1)

assert sys.version >= '2', "Install Python 2.0 or greater"
try:
    True
except:
    True = 1
    False = 0


def fmttime(n):
    if n <= 0:
        return None
    n = int(n)
    m, s = divmod(n, 60)
    h, m = divmod(m, 60)
    if h > 1000000:
        return 'connecting to peers'
    return 'downloading, ETA in %d:%02d:%02d' % (h, m, s)

def fmtsize(n):
    unit = [' B', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB']
    i = 0
    if (n > 999):
        i = 1
        while i + 1 < len(unit) and (n >> 10) >= 999:
            i += 1
            n >>= 10
        n = float(n) / (1 << 10)
    if i > 0:
        size = '%.1f' % n + '%s' % unit[i]
    else:
        size = '%.0f' % n + '%s' % unit[i]
    return size


class CursesDisplayer:
    def __init__(self, scrwin):
        self.scrwin = scrwin
        signal(SIGWINCH, self.winch_handler)
        self.changeflag = Event()
        self._remake_window()

    def winch_handler(self, signum, stackframe):
        self.changeflag.set()
        curses.endwin()
        self.scrwin.refresh()
        self.scrwin = curses.newwin(0, 0, 0, 0)
        self._remake_window(resize=True)

    def _remake_window(self, resize=False):
        self.scrh, self.scrw = self.scrwin.getmaxyx()
        self.scrpan = curses.panel.new_panel(self.scrwin)
        self.mainwinh = self.scrh - 5  # - 2 (bars) - 1 (debugwin) - 1 (borderwin) - 1 (totalwin)
        self.mainwinw = self.scrw - 4  # - 2 (bars) - 2 (spaces)
        self.mainwiny = 2         # + 1 (bar) + 1 (titles)
        self.mainwinx = 2         # + 1 (bar) + 1 (space)
        # + 1 to all windows so we can write at mainwinw

        self.mainwin = curses.newwin(self.mainwinh, self.mainwinw+1,
                                     self.mainwiny, self.mainwinx)
        self.mainpan = curses.panel.new_panel(self.mainwin)
        self.mainwin.scrollok(0)

        self.headerwin = curses.newwin(1, self.mainwinw+1,
                                       1, self.mainwinx)
        self.headerpan = curses.panel.new_panel(self.headerwin)
        self.headerwin.scrollok(0)

        self.totalwin = curses.newwin(1, self.mainwinw+1,
                                      self.scrh-3, self.mainwinx)
        self.totalpan = curses.panel.new_panel(self.totalwin)
        self.totalwin.scrollok(0)

        self.statuswin = curses.newwin(1, self.mainwinw+1,
                                       self.scrh-2, self.mainwinx)
        self.statuspan = curses.panel.new_panel(self.statuswin)
        self.statuswin.scrollok(0)
        if resize:
            self.statuswin.addstr(0, 0, 'window resize: %s x %s' %
                                  (self.scrw, self.scrh))
        else:
            self.statuswin.addstr(0, 0, 'btlaunchmany started')

        self.scrwin.border(ord('|'),ord('|'),ord('-'),ord('-'),ord(' '),ord(' '),ord(' '),ord(' '))
        self.headerwin.addnstr(0, 0, 'Filename', self.mainwinw - 25, curses.A_BOLD)
        self.headerwin.addnstr(0, self.mainwinw - 24, 'Size', 4);
        self.headerwin.addnstr(0, self.mainwinw - 18, 'Download', 8);
        self.headerwin.addnstr(0, self.mainwinw -  6, 'Upload', 6);
        self.totalwin.addnstr(0, self.mainwinw - 27, 'Totals:', 7);
        curses.panel.update_panels()
        curses.doupdate()
        self.changeflag.clear()


    def display(self, data):
        if self.changeflag.isSet():
            return
        self.mainwin.erase()
        winpos = 0
        totalup = 0
        totaldn = 0
        for x in data:
            ( name, status, progress, peers, seeds, seedsmsg,
              uprate, dnrate, upamt, dnamt, size, t ) = x
            t = fmttime(t)
            if t:
                status = t
            totalup += uprate
            totaldn += dnrate
            size = fmtsize(size)
            uprate = '%s/s' % fmtsize(uprate)
            dnrate = '%s/s' % fmtsize(dnrate)
            self.mainwin.addnstr(winpos, 0,
                                 name, self.mainwinw - 28, curses.A_BOLD)
            self.mainwin.addnstr(winpos, self.mainwinw-28 + (8-len(size)),
                                 size, 8)
            self.mainwin.addnstr(winpos, self.mainwinw-20 + (10-len(dnrate)),
                                 dnrate, 10)
            self.mainwin.addnstr(winpos, self.mainwinw-10 + (10-len(uprate)),
                                 uprate, 10)
            winpos = winpos + 1
            self.mainwin.addnstr(winpos, 0, '^--- ', 5)
            self.mainwin.addnstr(winpos, 6, status, self.mainwinw - 5)
            winpos = winpos + 1
        # display statusline
        self.statuswin.erase() 
#        self.statuswin.addnstr(0, 0, status, mainwinw)
        # display totals line
        totalup = '%s/s' % fmtsize(totalup)
        totaldn = '%s/s' % fmtsize(totaldn)
        
        self.totalwin.erase()
        self.totalwin.addnstr(0, self.mainwinw-27, 'Totals:', 7)
        self.totalwin.addnstr(0, self.mainwinw-20 + (10-len(totaldn)),
                              totaldn, 10)
        self.totalwin.addnstr(0, self.mainwinw-10 + (10-len(totalup)),
                              totalup, 10)
        curses.panel.update_panels()
        curses.doupdate()

    def message(self, s):
        pass


def LaunchManyWrapper(scrwin, config):
    outclass = CursesDisplayer(scrwin)
    LaunchMany(config, outclass)


if __name__ == '__main__':
    if argv[1:] == ['--version']:
        print version
        exit(0)
    defaults.append(
        ('parse_dir_interval', 60,
            "how often to rescan the torrent directory, in seconds")
    )
    try:
        config, args = parseargs(argv[1:], defaults, 1, 1)
        if not os.path.isdir(args[0]):
            raise ValueError("Warning: "+args[0]+" is not a directory")
        config['torrent_dir'] = args[0]
    except ValueError, e:
        print 'error: ' + str(e) + '\nrun with no args for parameter explanations'
        exit(1)
    if not config:
        print "Usage: btlaunchmany.py <directory> <global options>\n"
        print "<directory> - directory to look for .torrent files (semi-recursive)"
        print get_usage(defaults)
        exit(1)

    curses_wrapper(LaunchManyWrapper, config)